#!/usr/bin/env python
# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.

# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import mock
import imp
import subprocess

from ci_libs.ticket import Ticket
from tests import CLITestCase


class MainScriptTestCase(CLITestCase):

    def setUp(self):
        super(MainScriptTestCase, self).setUp()
        self.main_script = (os.path.join(os.path.dirname(__file__),
                            "../ubuntu-ci"))
        self.cli = imp.load_source(
            "ubuntu-ci",
            self.main_script,
        )

    def _run_cmdline(self, args):
        cmdline = "%s %s" % (self.main_script, ' '.join(args))
        process = subprocess.Popen(cmdline, shell=True, stdin=subprocess.PIPE,
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE)
        return process.communicate()

    @mock.patch('ci_libs.ticket.SubTicket._process')
    @mock.patch('ci_libs.utils.post',
                return_value='http://www.example.com/api/v1/ticket/38/')
    @mock.patch('ci_libs.utils.patch')
    def test_add_new_ticket(self, mock_patch, mock_post, mock_process):
        changes = os.path.join(os.path.dirname(__file__),
                               'data/foobar_0.1-1_source.changes')
        args = ['create_ticket', '-t', 'New feature', '-b', '1234', '-o',
                'someone@example.com', '-d', 'This is a cool new feature',
                '-r', 'baz', '-s', changes]

        new_ticket = Ticket()
        args = self.cli.parse_arguments(args)
        new_ticket.add_new_ticket(args)
        self.assertEqual(mock_process.call_count, 1)
        self.assertEqual(mock_post.call_count, 1)
        self.assertEqual(mock_patch.call_count, 1)
        self.assertEqual(len(new_ticket.subtickets), 1)
        self.assertTrue("foobar" in new_ticket.subtickets)

    @mock.patch('ci_libs.ticket.SubTicket._process')
    @mock.patch('ci_libs.utils.post',
                return_value='http://www.example.com/api/v1/ticket/39/')
    @mock.patch('ci_libs.utils.patch')
    def test_add_new_ticket_multiple_subtickets(self, mock_patch, mock_post,
                                                mock_process):
        changes_1 = os.path.join(os.path.dirname(__file__),
                                 'data/foobar_0.1-1_source.changes')
        changes_2 = os.path.join(os.path.dirname(__file__),
                                 'data/barfoo_0.1-1_source.changes')

        args = ['create_ticket', '-t', '"New feature"', '-b', '4321', '-o',
                'someone@example.com', '-d', '"This is a cool new feature"',
                '-r', 'baz', '-s', changes_1, '-s', changes_2]

        new_ticket = Ticket()
        args = self.cli.parse_arguments(args)
        new_ticket.add_new_ticket(args)
        self.assertEqual(mock_process.call_count, 2)
        self.assertEqual(mock_post.call_count, 1)
        self.assertEqual(mock_patch.call_count, 1)

        self.assertEqual(len(new_ticket.subtickets), 2)
        self.assertTrue("foobar" in new_ticket.subtickets)
        self.assertTrue("barfoo" in new_ticket.subtickets)

    def test_fields_are_required(self):
        args = ['create_ticket']
        (output, error) = self._run_cmdline(args)
        self.assertIsNotNone(error)
        self.assertTrue(
            "create_ticket: error: argument -t/--title is required" in error)

        args.extend(['-t', '"New feature"'])
        (output, error) = self._run_cmdline(args)
        self.assertIsNotNone(error)
        self.assertTrue(
            "create_ticket: error: argument -d/--description is required"
            in error)

        args.extend(['-d', '"This is a cool new feature"'])
        (output, error) = self._run_cmdline(args)
        self.assertIsNotNone(error)
        self.assertTrue(
            "create_ticket: error: argument -o/--owner is required"
            in error)

        args.extend(['-o', 'someone@example.com'])
        (output, error) = self._run_cmdline(args)
        self.assertIsNotNone(error)
        self.assertTrue(
            "create_ticket: error: argument -s/--sources is required"
            in error)

        args.extend(['-s', 'foobar.changes'])
        (output, error) = self._run_cmdline(args)
        self.assertIsNotNone(error)
        # All required arguments were provided. Intentionally failing here to
        # avoid having to mock post.
        self.assertTrue("ERROR:ubuntu-ci:Changes file not found" in error)
